import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class AppTextField extends StatefulWidget {
  const AppTextField({
    Key? key,
    this.labelText,
    this.hintText,
    this.errorText = '',
    this.helpText,
    this.counterText = '',
    this.textStyle,
    this.borderColor,
    this.backgroundColor,
    this.minLines,
    this.maxLines = 1,
    this.maxLength,
    this.obscureText = false,
    this.enable = true,
    this.autoFocus = false,
    this.onTap,
    this.keyboardType = TextInputType.text,
    this.inputFormatters,
    this.focusNode,
    this.controller,
    this.onChangedText,
    this.onChangedFocus,
    this.onSubmit,
    this.isSearch,
    this.prefixIcon,
    this.readOnly = false,
    this.textAlign = TextAlign.left,
    this.onEditingComplete,
    this.textCapitalization = TextCapitalization.none,
    this.autocorrect = true,
    this.enableSuggestions = true,
    this.height,
    this.isArea,
    this.searchTap,
    this.textInputAction,
    this.suffixIcon,
  }) : super(key: key);

  final String? labelText;
  final String? hintText;
  final String errorText;
  final String? helpText;
  final String? counterText;
  final TextStyle? textStyle;
  final Color? borderColor;
  final Color? backgroundColor;
  final int? minLines;
  final int maxLines;
  final int? maxLength;
  final double? height;
  final bool? isArea;
  final bool? isSearch;
  final bool obscureText;
  final bool enable;
  final bool autoFocus;
  final Function()? onTap;
  final TextInputType keyboardType;
  final List<TextInputFormatter>? inputFormatters;
  final FocusNode? focusNode;
  final TextEditingController? controller;
  final ValueChanged<String>? onChangedText;
  final Function(bool?)? onChangedFocus;
  final ValueChanged<String>? onSubmit;
  final bool readOnly;
  final TextAlign textAlign;
  final Function()? onEditingComplete;
  final Function()? searchTap;
  final TextCapitalization textCapitalization;
  final bool autocorrect;
  final bool enableSuggestions;
  final TextInputAction? textInputAction;
  final Widget? prefixIcon;
  final Widget? suffixIcon;

  @override
  AppTextFieldState createState() => AppTextFieldState();
}

class AppTextFieldState extends State<AppTextField> {
  dynamic _focusNode;
  bool _obscureText = true;

  @override
  void initState() {
    if (widget.focusNode != null) {
      _focusNode = widget.focusNode;
    } else {
      _focusNode = FocusNode();
    }
    _focusNode.addListener(() {
      if (widget.onChangedFocus != null) {
        widget.onChangedFocus!(_focusNode.hasFocus);
      }
      if (mounted) {
        setState(() {});
      }
    });
    super.initState();
  }

  onChanged(value) {
    if (widget.onChangedText != null) widget.onChangedText!(value);
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Container(
            height: (widget.isArea ?? false) ? 100 : 45,
            decoration: BoxDecoration(
              color: Colors.white,
              borderRadius: BorderRadius.circular(5),
              border: Border.all(
                  color: (widget.errorText.isEmpty
                      ? widget.borderColor
                      : Colors.red) ??
                      (_focusNode.hasFocus
                          ? Colors.purple
                          : Colors.grey),
                  width: 1.0),
            ),
            child: Stack(
              children: [
                TextField(
                  textInputAction: widget.textInputAction,
                  focusNode: _focusNode,
                  style: widget.textStyle,
                  enableSuggestions: widget.enableSuggestions,
                  autocorrect: widget.autocorrect,
                  textCapitalization: widget.textCapitalization,
                  onEditingComplete: widget.onEditingComplete,
                  textAlign: widget.textAlign,
                  keyboardType: widget.keyboardType,
                  readOnly: widget.readOnly,
                  inputFormatters: widget.inputFormatters,
                  autofocus: widget.autoFocus,
                  onTap: widget.onTap,
                  cursorColor: Colors.black,
                  maxLength: widget.maxLength,
                  maxLines: (widget.isArea ?? false) ? null : 1,
                  controller: widget.controller,
                  enabled: widget.enable,
                  obscureText:
                  widget.obscureText ? _obscureText : widget.obscureText,
                  decoration: InputDecoration(
                    counterText: widget.counterText,
                    labelText: widget.labelText,
                    hintText: widget.hintText,
                    helperText: widget.helpText,
                    contentPadding: EdgeInsets.symmetric(
                        horizontal: 16,
                        vertical: 16),
                    prefixIcon: widget.prefixIcon,
                    suffixIcon: widget.obscureText
                        ? IconButton(
                      iconSize: 25,
                      icon: Icon(
                        _obscureText
                            ? Icons.visibility
                            : Icons.visibility_off,
                        color: Colors.orange,
                      ),
                      onPressed: () {
                        setState(() {
                          _obscureText = !_obscureText;
                        });
                      },
                    )
                        : widget.suffixIcon,
                    prefixIconConstraints: const BoxConstraints(
                      minWidth: 20,
                      minHeight: 20,
                    ),
                    enabledBorder: InputBorder.none,
                    disabledBorder: InputBorder.none,
                    focusedBorder: InputBorder.none,
                    errorBorder: InputBorder.none,
                    focusedErrorBorder: InputBorder.none,
                    border: InputBorder.none,
                  ),
                  onChanged: onChanged,
                  onSubmitted: widget.onSubmit,
                ),
                if (widget.isSearch ?? false)
                  SizedBox(
                    width: double.infinity,
                    child: InkWell(
                      onTap: () {
                        widget.searchTap!();
                      },
                      child: Container(width: double.infinity,
                      ),
                    ),
                  )
              ],
            )),

        //errorText element
        if (widget.errorText.isNotEmpty)
          Padding(
            padding: EdgeInsets.only(top: 4),
            child: Text(
              widget.errorText,
            ),
          )
      ],
    );
  }

}
